﻿using System;
using System.Collections.Generic;
using System.Linq;
using Domain;
using Domain.Dtos;
using McMaster.Extensions.CommandLineUtils;

namespace ConsolePasswordAuth
{
    class Program
    {
        const string API_URL = "https://api-no.drofus.com/api";

        static void Main(string[] args)
        {
            var app = new CommandLineApplication();

            app.HelpOption("-?|--help");
            var usernameOption = app.Option("-u|--username <VALUE>", "dRofus username", CommandOptionType.SingleValue);
            var passwordOption = app.Option("-p|--password <VALUE>", "dRofus user password", CommandOptionType.SingleValue);
            var databaseOption = app.Option("-d|--database <VALUE>", "dRofus database", CommandOptionType.SingleValue);
            var projectIdOption = app.Option("-i|--project-id <VALUE>", "Optional project id, must be provided if more than one project in the database", CommandOptionType.SingleValue);
            var topOption = app.Option<int>("-t|--top <VALUE>", "Optional limit to how many rooms to get. Defaults to 10", CommandOptionType.SingleValue);
            
            app.OnExecuteAsync(async (ct) =>
            {

                var username = usernameOption.HasValue() ? usernameOption.Value() : Prompt.GetString("Username:");
                var password = passwordOption.HasValue() ? passwordOption.Value() : Prompt.GetPassword("Password");
                var database = databaseOption.HasValue() ? databaseOption.Value() : Prompt.GetString("Database:");
                var projectId = projectIdOption.HasValue() ? projectIdOption.Value() : string.Empty;
                var top = topOption.HasValue() ? topOption.ParsedValue : 10;

                if (new[] { username, password, database}.Any(x => string.IsNullOrEmpty(x)))
                {
                    Console.WriteLine("Not all required options provided:");
                    Console.WriteLine(app.GetHelpText());
                    return;
                }

                var loginUsername = string.Join("/", new[] { database, projectId, username }.Where(x => !string.IsNullOrEmpty(x)));
                var rooms = await new GetRoomsReader(API_URL, loginUsername, password).GetAsync(top);
                PrintRooms(rooms.ToList());

                return;
            });

            app.Execute(args);
        }

        private static void PrintRooms(IEnumerable<Room> rooms)
        {
            Console.WriteLine($" {"Id",-5}   {"Room Function #",-15}   {"Name",-50}   {"Programmed area",-15}");
            Console.WriteLine("-----------------------------------------------------------------------------------------------");

            foreach (var room in rooms)
            {
                Console.WriteLine($" {room.Id,-5} | {room.FunctionNumber,-15} | {room.FullName,-50} | {room.ProgrammedArea,15}");
            }
        }
    }
}
