﻿using System;
using System.Collections.Generic;
using System.Net.Http;
using System.Net.Http.Headers;
using System.Text;
using System.Threading.Tasks;
using Newtonsoft.Json;

namespace Domain
{
    public abstract class Reader<T> where T : Dtos.ParametricItem
    {
        private readonly string _apiUrl;
        private readonly string _accessToken;
        private readonly string _username;
        private readonly string _password;

        public Reader(string apiUrl, string accessToken)
        {
            _apiUrl = apiUrl;
            _accessToken = accessToken;
        }

        /// <param name="databaseAndUsername">on format [database]/[projectId (optional if only one project)]/[username]</param>
        /// <param name="password"></param>
        public Reader(string apiUrl, string databaseAndUsername, string password)
        {
            _apiUrl = apiUrl;
            _username = databaseAndUsername;
            _password = password;
        }

        protected string ApiUrl { get; }

        protected abstract string GetCallUrl();

        public async Task<IEnumerable<T>> GetAsync(int? top = null)
        {
            var requestUrl = $"{_apiUrl}{GetCallUrl()}";
            if (top.HasValue)
            {
                requestUrl += $"&$top={top}";
            }

            var client = new HttpClient();
            client.DefaultRequestHeaders.Accept.Clear();
            client.DefaultRequestHeaders.Authorization = GetAuthenticationHeader();

            var json = await client.GetStringAsync(requestUrl);
            var dtos = JsonConvert.DeserializeObject<IEnumerable<T>>(json);
            return dtos;
        }

        private AuthenticationHeaderValue GetAuthenticationHeader()
        {
            return !string.IsNullOrEmpty(_accessToken) ? GetAccessTokenHeader(_accessToken) : GetBasicAuthHeader(_username, _password);
        }

        private static AuthenticationHeaderValue GetAccessTokenHeader(string accessToken)
        {
            return new AuthenticationHeaderValue("Bearer", accessToken);
        }

        private static AuthenticationHeaderValue GetBasicAuthHeader(string username, string password)
        {
            var auth = GetBase64($"{username}:{password}");
            return new AuthenticationHeaderValue("Basic", auth);
        }

        static string GetBase64(string input)
        {
            var encodedBytes = Encoding.UTF8.GetBytes(input);
            return Convert.ToBase64String(encodedBytes);
        }

    }
}
