﻿using Domain;
using System;
using System.Collections.Generic;
using System.Threading.Tasks;
using System.Windows;
using WpfLoginApi.Login;

namespace WpfLoginApi
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private string _accessToken;
        private const string IDENTITY_SERVER_URL = "https://ids-db2.drofus.com";
        private const string API_URL = "https://api-no.drofus.com/api";

        public MainWindow()
        {
            InitializeComponent();
            this.DataContext = this;
        }

        public IEnumerable<DummyItem> Rooms => GetDummyRooms();

        public DummyItem SelectedRoom { set { this.BuildLookupUrlAndNavigate(value?.Number, "room"); } }

        public IEnumerable<DummyItem> Occurrences => GetDummyOccurrences();

        public DummyItem SelectedOccurrence { set { this.BuildLookupUrlAndNavigate(value?.Number, "occurrence"); } }
        
        private void BuildLookupUrlAndNavigate(string lookupValue, string type)
        {
            if (lookupValue != null)
            {
                var url = $"{this.Server.Text}/{this.DatabaseProject.Text}/#external?type={type}&value={lookupValue}";
                this.Message.Text = $"Navigating to: {url}";
                Browser.Navigate(new Uri(url));
            }
            else
            {
                this.Message.Text = $"Please select a room";
            }
        }
        
        private IEnumerable<DummyItem> GetDummyRooms()
        {
            return new[] 
            {
                new DummyItem { Id = 123, Number = "01.001", Name = "Office, small" },
                new DummyItem { Id = 124, Number = "01.004", Name = "Office, small" },
            };
        }

        private IEnumerable<DummyItem> GetDummyOccurrences()
        {
            return new[]
            {
                new DummyItem { Id = 987, Number = "+XX=360.001", Name = "Air ventilation" },
                new DummyItem { Id = 653, Number = "+XX=360.001-JP001", Name = "Pumpe" },
            };
        }

        public class DummyItem
        {
            public long Id { get; set; }
            public string Number { get; set; }
            public string Name { get; set; }
        }
    }
}
