﻿using System;
using System.Net.Http;
using System.Threading.Tasks;
using IdentityModel.Client;
using IdentityModel.OidcClient;
using Microsoft.AspNetCore.WebUtilities;

namespace WpfLoginApi.Login
{
    public class GetTokenService
    {
        public async Task<Result> GetTokenAsync(string identityUrl)
        {
            var discovery = await GetDiscoveryDocument(identityUrl);

            var options = new OidcClientOptions()
            {
                Authority = identityUrl,
                ClientId = "drofus-api",
                Scope = "drofus-api",
                RedirectUri = "http://localhost:8888/landing",
                ResponseMode = OidcClientOptions.AuthorizeResponseMode.Redirect,
                Flow = OidcClientOptions.AuthenticationFlow.AuthorizationCode,
                ClientSecret = "e4HdT5m6F7nEB7wqgY_qBqctJS+TTbPVWGpvjry!HG3R9t/Sg2cz"
            };

            var oidcClient = new OidcClient(options);
            var state = await oidcClient.PrepareLoginAsync();
            
            var browser = new WpfEmbeddedBrowser();
            var redirectQuery = await browser.GetRedirectQueryAsync(state.StartUrl, options.RedirectUri);
            if (string.IsNullOrEmpty(redirectQuery))
            {
                return Result.Error("Error logging in in browser");
            }

            var queryParams = QueryHelpers.ParseQuery(redirectQuery);
            if (!queryParams.TryGetValue("code", out var codeVal))
            {
                return Result.Error("Getting Code from redirect query");
            }
            
            var client = new HttpClient();
            var response = await client.RequestAuthorizationCodeTokenAsync(
                new AuthorizationCodeTokenRequest
                {
                    Address = discovery.TokenEndpoint,
                    ClientId = options.ClientId,
                    ClientSecret = options.ClientSecret,
                    Code = codeVal,
                    RedirectUri = options.RedirectUri, 
                    CodeVerifier = state.CodeVerifier
                });

            if (response.IsError)
            {
                return Result.Error(response.Error);
            }

            return Result.Success(response.AccessToken);
        }

        private async Task<DiscoveryDocumentResponse> GetDiscoveryDocument(string identityAddress)
        {
            var client = new HttpClient();

            var discoveryDocument = await client.GetDiscoveryDocumentAsync(identityAddress);
            if (discoveryDocument.IsError)
            {
                throw new Exception(discoveryDocument.Error);
            }

            return discoveryDocument;
        }

        public class Result
        {
            private Result()
            {
            }

            public string AccessToken { get; private set; } = string.Empty;
            public string ErrorMessage { get; private set; } = string.Empty;
            public bool IsError => !string.IsNullOrEmpty(ErrorMessage);

            public static Result Error(string errorMsg)
            {
                return new Result { ErrorMessage = errorMsg };
            }

            public static Result Success(string accessToken)
            {
                return new Result { AccessToken = accessToken };
            }
        }
    }
}
