﻿using Domain;
using System;
using System.Threading.Tasks;
using System.Windows;
using WpfLoginApi.Login;

namespace WpfLoginApi
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private string _accessToken;
        private const string IDENTITY_SERVER_URL = "https://ids-db2.drofus.com";
        private const string API_URL = "https://api-no.drofus.com/api";

        public MainWindow()
        {
            InitializeComponent();
        }

        private async void LoginButton_Click(object sender, RoutedEventArgs e)
        {
            await DoLogin();
        }

        private async void CallApiButton_Click(object sender, RoutedEventArgs e)
        {
            if (string.IsNullOrEmpty(_accessToken))
            {
                await DoLogin();
                if (string.IsNullOrEmpty(_accessToken))
                {
                    // give up
                    return;
                }
            }

            try
            {
                // get rooms:
                var data = await new GetRoomsReader(API_URL, _accessToken).GetAsync();
                RoomList.ItemsSource = data;
            }
            catch (Exception ex)
            {
                Message.Text = ex.Message;
            }
        }

        private async Task DoLogin()
        {
            var result = await new GetTokenService().GetTokenAsync(IDENTITY_SERVER_URL);
            if (result.IsError)
            {
                _accessToken = string.Empty;
                Message.Text = result.ErrorMessage;
            }
            else
            {
                _accessToken = result.AccessToken;
                Message.Text = "Authorized :)";
            }

        }
    }
}
